package com.diagbot.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.dto.GetConsoleOnTrialDTO;
import com.diagbot.dto.GetConsoleOpenedDTO;
import com.diagbot.dto.UserAndProdutDTO;
import com.diagbot.entity.OpenedProducts;
import com.diagbot.entity.OpenedProductsIndex;
import com.diagbot.entity.wrapper.OpendProductWrapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 已开通产品 Mapper 接口
 * </p>
 *
 * @author gaodm
 * @since 2018-09-18
 */
public interface OpenedProductsMapper extends BaseMapper<OpenedProducts> {
    /**
     * 根据用户id查询已开通的产品
     *
     * @param userId 用户id
     * @return
     */
    IPage<OpenedProducts> selectOpendInfoByUserId(Page page, @Param("userId") int userId);

    /**
     * 查询已开通产品
     *
     * @param map 查询条件
     * @return
     */
    List<OpenedProducts> selectOpenedProducts(Map<String, Object> map);


    /**
     * 查询已开通产品
     *
     * @param map 查询条件
     * @return
     */
    List<OpenedProducts> getByAppkeyAndSecret(Map<String, Object> map);


    /**
     * 控制台已开通产品查询
     *
     * @param userId
     * @return
     */
    List<GetConsoleOpenedDTO> getConsoleOpened(Long userId);

    /**
     * 控制台已开通产品数量查询
     *
     * @param userId
     * @return
     */
    int getConsoleOpenedCount(Long userId);

    /**
     * 控制台可试用产品查询
     *
     * @param userId
     * @return
     */
    List<GetConsoleOnTrialDTO> getConsoleOnTrial(Long userId);

    /**
     * 控制台可试用产品数量查询
     *
     * @param userId
     * @return
     */
    int getConsoleOnTrialCount(Long userId);

    //根据用户id查询已开通的功能
    public List<UserAndProdutDTO> getInformationAvailableByUserId(OpenedProducts openedProducts);

    //根据用户id和产品id，启用和停用功能
    public boolean startAndendByuserId(OpenedProducts openedProducts);

    //根据用户id和产品id，删除开通信息", notes = "删除开通信息")
    public boolean delInformationAvailable(OpenedProducts openedProducts);

    //根据用户id和产品id，修改开通时间", notes = "修改开通时间")
    public boolean modifyOpeningTime(OpenedProducts openedProducts);


    /**
     * @Description: 根据产品id查询所有开通用户信息
     * @Author: wangyu
     * @Date: 20:23 2018/9/20
     */
    public List<OpendProductWrapper> getByProductId(OpenedProductsIndex openedProductsIndex);

    /**
     * @Description: 根据产品id查询该产品是否有用户开通
     * @Author: wangyu
     * @Date: 20:23 2018/9/20
     */
    public List<OpenedProducts> getOpendById(OpenedProducts openedProducts);
}
